function [Output] = enz_cent_Lib_dir_quatexelero(fileName1,fileName2)
% Builds Directed Enzyme-Enzyme Network with Removing Currency Metabolites (based-on a Library file) which could be used as an input for QuateXelero Algorithm.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The function reads a Metabolic Network SBML file and builds an Directed Enzyme-Enzyme Network which is compatible with QuateXelero Algorithm.
% For every metabolite, the algorithm checks availability in the Library file which has been prepared by user as input in .txt format).
% and removes if it exists in the library file. Then the Directed Enzyme-Enzyme Network will be created.
% The QuateXelero is one of the best motif finding algorithms which is recently developed by Kavosh developer team.
% http://lbb.ut.ac.ir/Download/LBBsoft/QuateXelero
% So, one could easily use this algorithm in order to find motifs in different sizes for the metabolic network.
% Note: COBRA Toolbox must be installed in MATLAB before running this function
%
% [Output] = enz_cent_Lib_dir_quatexelero(fileName1,fileName2)
%
%INPUTS
% fileName1                                The Library file includes pre-defined currency metabolites (in .txt format)
% Note: Library text file must include one metabolites per line (all in one column) 
% fileName2                                The metabolic Network in the SBML format
% 
%OUTPUTS
% *_Enzyme_Cent_Lib_Dir_Index.dat          Matrix Indeces of Enzyme-Enzyme Connections 
% *_Enzyme_Cent_Lib_Dir_QuateXelero.dat    Directed-Enzyme-Enzyme Network - QuateXelero Compatible
% 
% Yazdan Asgari 07/16/2016         http://yazdan59.ir/scan
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% check validity of input files format
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
check1=regexp(fileName1,'.txt');
assert(~isempty(check1),'Error in the first input: The fileName1 must contain .txt at its end')
check2=regexp(fileName2,'.xml');
assert(~isempty(check2),'Error in the second input: The fileName2 must contain .xml at its end')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% start time evaluation of program
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% reading the Library text file and construct array of currency metabolites
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fid = fopen(fileName1);
tline = fgetl(fid);
i=1;
Curr_met={};
while ischar(tline)
    Curr_met{i,1}=tline;
    tline = fgetl(fid);
    i=i+1;
end
fclose(fid);
[h,g]=size(Curr_met);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% reading the SBML file using COBRA Toolbox Command, and sets size of the S matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
model=readCbModel(fileName2);
[m,n]=size(model.S);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% reading the Metabolites array and check their availability in the library text file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N_curr=zeros(m,1);
for q=1:m
    for i=1:h
        if strcmp(model.metNames{q},Curr_met{i,1})==1
            N_curr(q,1)=N_curr(q,1)+1;
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remove metabolites which are in the input Currecny Metabolites list
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for q=1:m
    if N_curr(q,1)~=0
        for i=1:n
            model.S(q,i)=0;
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% building the output file name
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
outname1=strrep(fileName2,'.xml','_Enzyme_Cent_Lib_Dir_Index.dat')
fout1 = fopen(outname1, 'w+');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% finds non-zero elements of the S-matrix (in order to make the algorithm faster),
% parses through each row, and considers an edge for every unlike-signs.
% It also consider Reversibility for Enzyme-Enzyme network.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
num=zeros(size(model.rxns));
for j=1:m
    indices=find(model.S(j,:));
    [a,b]=size(indices);
    r=0;
    if b~=0
        r=1;
    end
    while r<b
        i=1;
        while i<(b-r+1)
            if model.S(j,indices(1,r))>0 && model.S(j,indices(1,r+i))<0
                if model.rev(indices(r+i))==1 && model.rev(indices(r))==1
                    fprintf(fout1,'%d\t%d\n',indices(1,r),indices(1,r+i));
                    fprintf(fout1,'%d\t%d\n',indices(1,r+i),indices(1,r));
                    num(1,indices(1,r))=1;
                    num(1,indices(1,r+i))=1;
                else
                    fprintf(fout1,'%d\t%d\n',indices(1,r),indices(1,r+i));
                    num(1,indices(1,r))=1;
                    num(1,indices(1,r+i))=1;
                end
            elseif model.S(j,indices(1,r))<0 && model.S(j,indices(1,r+i))>0
                if model.rev(indices(r+i))==1 && model.rev(indices(r))==1
                    fprintf(fout1,'%d\t%d\n',indices(1,r+i),indices(1,r));
                    fprintf(fout1,'%d\t%d\n',indices(1,r),indices(1,r+i));
                    num(1,indices(1,r))=1;
                    num(1,indices(1,r+i))=1;
                else
                    fprintf(fout1,'%d\t%d\n',indices(1,r+i),indices(1,r));
                    num(1,indices(1,r))=1;
                    num(1,indices(1,r+i))=1;
                end
            end
            i=i+1;
        end
        r=r+1;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% building the output file name
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
outname2=strrep(fileName2,'.xml','_Enzyme_Cent_Dir_QuateXelero.dat')    
fout2=fopen(outname2,'w+');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% reading the constructed Enzyme-Enzyme network file and re-format it to a QuateXelero-compatible file.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fid = fopen(outname1);
C=fscanf(fid,'%d');
i=1;
while isinteger(fid)
    C(i)=fscanf(fid,'%d',C);
    i=i+1;
end
g=size(C);
A=size(unique(C));
if g~=0
    n=1;
else
    disp('Error in reading the file, No Edge detected')
end
k=1;
j=2;
last=g/2;
fprintf(fout2,'%d\n',A(1,1));   % total number of uniques nodes in the network (needed for QuateXelero Algorithm)
for i=1:last
    fprintf(fout2,'%d\t%d\n ',C(k),C(j));
    k=k+2;
    j=j+2;
end
fclose(fid);
fclose(fout2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% End of time evaluation of program
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
toc;
